@@ -102,13 +102,16 @@ void TlsRecordFilter::SecretCallback(PRFileDesc* fd, PRUint16 epoch,
             SSL_GetCipherSuiteInfo(suite, &cipherinfo, sizeof(cipherinfo)));
   EXPECT_EQ(sizeof(cipherinfo), cipherinfo.length);
 
-  bool is_dtls = self->agent()->variant() == ssl_variant_datagram;
-  self->cipher_specs_.emplace_back(is_dtls, epoch);
+  self->cipher_specs_.emplace_back(self->is_dtls_agent(), epoch);
   EXPECT_TRUE(self->cipher_specs_.back().SetKeys(&cipherinfo, secret));
 }
 
+bool TlsRecordFilter::is_dtls_agent() const {
+  return agent()->variant() == ssl_variant_datagram;
+}
+
 bool TlsRecordFilter::is_dtls13() const {
-  if (agent()->variant() != ssl_variant_datagram) {
+  if (!is_dtls_agent()) {
     return false;
   }
   if (agent()->state() == TlsAgent::STATE_CONNECTED) {
@@ -137,8 +140,7 @@ TlsCipherSpec& TlsRecordFilter::spec(uint16_t write_epoch) {
   // count sequence numbers.
   EXPECT_FALSE(decrypting_) << "No spec available for epoch " << write_epoch;
   ;
-  bool is_dtls = agent()->variant() == ssl_variant_datagram;
-  cipher_specs_.emplace_back(is_dtls, write_epoch);
+  cipher_specs_.emplace_back(is_dtls_agent(), write_epoch);
   return cipher_specs_.back();
 }
 
@@ -504,7 +506,7 @@ bool TlsRecordFilter::Unprotect(const TlsRecordHeader& header,
   if (!decrypting_ || !header.is_protected()) {
     // Maintain the epoch and sequence number for plaintext records.
     uint16_t ep = 0;
-    if (agent()->variant() == ssl_variant_datagram) {
+    if (is_dtls_agent()) {
       ep = static_cast<uint16_t>(header.sequence_number() >> 48);
     }
     spec(ep).RecordUnprotected(header.sequence_number());
@@ -515,7 +517,7 @@ bool TlsRecordFilter::Unprotect(const TlsRecordHeader& header,
   }
 
   uint16_t ep = 0;
-  if (agent()->variant() == ssl_variant_datagram) {
+  if (is_dtls_agent()) {
     ep = static_cast<uint16_t>(header.sequence_number() >> 48);
     if (!spec(ep).Unprotect(header, ciphertext, plaintext, out_header)) {
       return false;
@@ -1205,6 +1207,14 @@ PacketFilter::Action TlsClientHelloVersionSetter::FilterHandshake(
   return CHANGE;
 }
 
+PacketFilter::Action TlsServerHelloVersionSetter::FilterHandshake(
+    const HandshakeHeader& header, const DataBuffer& input,
+    DataBuffer* output) {
+  *output = input;
+  output->Write(0, version_, 2);
+  return CHANGE;
+}
+
 PacketFilter::Action SelectedCipherSuiteReplacer::FilterHandshake(
     const HandshakeHeader& header, const DataBuffer& input,
     DataBuffer* output) {
@@ -1221,4 +1231,43 @@ PacketFilter::Action SelectedCipherSuiteReplacer::FilterHandshake(
   output->Write(pos, static_cast<uint32_t>(cipher_suite_), 2);
   return CHANGE;
 }
+
+PacketFilter::Action ServerHelloRandomChanger::FilterHandshake(
+    const HandshakeHeader& header, const DataBuffer& input,
+    DataBuffer* output) {
+  *output = input;
+  uint32_t temp = 0;
+  size_t pos = 30;
+  EXPECT_TRUE(input.Read(pos, 2, &temp));
+  output->Write(pos, (temp ^ 0xffff), 2);
+  return CHANGE;
+}
+
+PacketFilter::Action ClientHelloPreambleCapture::FilterHandshake(
+    const HandshakeHeader& header, const DataBuffer& input,
+    DataBuffer* output) {
+  EXPECT_TRUE(header.handshake_type() == kTlsHandshakeClientHello);
+
+  if (captured_) {
+    return KEEP;
+  }
+  captured_ = true;
+
+  DataBuffer temp;
+  TlsParser parser(input);
+  EXPECT_TRUE(parser.Read(&temp, 2 + 32));     // Version + Random
+  EXPECT_TRUE(parser.ReadVariable(&temp, 1));  // Session ID
+  if (is_dtls_agent()) {
+    EXPECT_TRUE(parser.ReadVariable(&temp, 1));  // Cookie
+  }
+  EXPECT_TRUE(parser.ReadVariable(&temp, 2));  // Ciphersuites
+  EXPECT_TRUE(parser.ReadVariable(&temp, 1));  // Compression
+
+  // Copy the preamble into a new buffer
+  data_ = input;
+  data_.Truncate(parser.consumed());
+
+  return KEEP;
+}
+
 }  // namespace nss_test
