@@ -15,6 +15,7 @@
 #include "sslimpl.h"
 #include "sslproto.h"
 #include "ssl3exthandle.h"
+#include "tls13ech.h"
 #include "tls13err.h"
 #include "tls13exthandle.h"
 #include "tls13subcerts.h"
@@ -81,6 +82,7 @@ static const ssl3ExtensionHandler serverHelloHandlersTLS[] = {
 static const ssl3ExtensionHandler helloRetryRequestHandlers[] = {
     { ssl_tls13_key_share_xtn, tls13_ClientHandleKeyShareXtnHrr },
     { ssl_tls13_cookie_xtn, tls13_ClientHandleHrrCookie },
+    { ssl_tls13_encrypted_client_hello_xtn, tls13_ClientHandleHrrEchXtn },
     { 0, NULL }
 };
 
@@ -120,35 +122,34 @@ static const ssl3ExtensionHandler certificateRequestHandlers[] = {
  * the client hello is empty (for example, the extended master secret
  * extension, if it were listed last). See bug 1243641.
  */
-static const sslExtensionBuilder clientHelloSendersTLS[] =
-    {
-      { ssl_server_name_xtn, &ssl3_ClientSendServerNameXtn },
-      { ssl_extended_master_secret_xtn, &ssl3_SendExtendedMasterSecretXtn },
-      { ssl_renegotiation_info_xtn, &ssl3_SendRenegotiationInfoXtn },
-      { ssl_supported_groups_xtn, &ssl_SendSupportedGroupsXtn },
-      { ssl_ec_point_formats_xtn, &ssl3_SendSupportedPointFormatsXtn },
-      { ssl_session_ticket_xtn, &ssl3_ClientSendSessionTicketXtn },
-      { ssl_app_layer_protocol_xtn, &ssl3_ClientSendAppProtoXtn },
-      { ssl_use_srtp_xtn, &ssl3_ClientSendUseSRTPXtn },
-      { ssl_cert_status_xtn, &ssl3_ClientSendStatusRequestXtn },
-      { ssl_delegated_credentials_xtn, &tls13_ClientSendDelegatedCredentialsXtn },
-      { ssl_signed_cert_timestamp_xtn, &ssl3_ClientSendSignedCertTimestampXtn },
-      { ssl_tls13_key_share_xtn, &tls13_ClientSendKeyShareXtn },
-      { ssl_tls13_early_data_xtn, &tls13_ClientSendEarlyDataXtn },
-      /* Some servers (e.g. WebSphere Application Server 7.0 and Tomcat) will
+static const sslExtensionBuilder clientHelloSendersTLS[] = {
+    { ssl_server_name_xtn, &ssl3_ClientSendServerNameXtn },
+    { ssl_extended_master_secret_xtn, &ssl3_SendExtendedMasterSecretXtn },
+    { ssl_renegotiation_info_xtn, &ssl3_SendRenegotiationInfoXtn },
+    { ssl_supported_groups_xtn, &ssl_SendSupportedGroupsXtn },
+    { ssl_ec_point_formats_xtn, &ssl3_SendSupportedPointFormatsXtn },
+    { ssl_session_ticket_xtn, &ssl3_ClientSendSessionTicketXtn },
+    { ssl_app_layer_protocol_xtn, &ssl3_ClientSendAppProtoXtn },
+    { ssl_use_srtp_xtn, &ssl3_ClientSendUseSRTPXtn },
+    { ssl_cert_status_xtn, &ssl3_ClientSendStatusRequestXtn },
+    { ssl_delegated_credentials_xtn, &tls13_ClientSendDelegatedCredentialsXtn },
+    { ssl_signed_cert_timestamp_xtn, &ssl3_ClientSendSignedCertTimestampXtn },
+    { ssl_tls13_key_share_xtn, &tls13_ClientSendKeyShareXtn },
+    { ssl_tls13_early_data_xtn, &tls13_ClientSendEarlyDataXtn },
+    /* Some servers (e.g. WebSphere Application Server 7.0 and Tomcat) will
        * time out or terminate the connection if the last extension in the
        * client hello is empty. They are not intolerant of TLS 1.2, so list
        * signature_algorithms at the end. See bug 1243641. */
-      { ssl_tls13_supported_versions_xtn, &tls13_ClientSendSupportedVersionsXtn },
-      { ssl_signature_algorithms_xtn, &ssl3_SendSigAlgsXtn },
-      { ssl_tls13_cookie_xtn, &tls13_ClientSendHrrCookieXtn },
-      { ssl_tls13_psk_key_exchange_modes_xtn, &tls13_ClientSendPskModesXtn },
-      { ssl_tls13_post_handshake_auth_xtn, &tls13_ClientSendPostHandshakeAuthXtn },
-      { ssl_record_size_limit_xtn, &ssl_SendRecordSizeLimitXtn },
-      /* The pre_shared_key extension MUST be last. */
-      { ssl_tls13_pre_shared_key_xtn, &tls13_ClientSendPreSharedKeyXtn },
-      { 0, NULL }
-    };
+    { ssl_tls13_supported_versions_xtn, &tls13_ClientSendSupportedVersionsXtn },
+    { ssl_signature_algorithms_xtn, &ssl3_SendSigAlgsXtn },
+    { ssl_tls13_cookie_xtn, &tls13_ClientSendHrrCookieXtn },
+    { ssl_tls13_psk_key_exchange_modes_xtn, &tls13_ClientSendPskModesXtn },
+    { ssl_tls13_post_handshake_auth_xtn, &tls13_ClientSendPostHandshakeAuthXtn },
+    { ssl_record_size_limit_xtn, &ssl_SendRecordSizeLimitXtn },
+    /* The pre_shared_key extension MUST be last. */
+    { ssl_tls13_pre_shared_key_xtn, &tls13_ClientSendPreSharedKeyXtn },
+    { 0, NULL }
+};
 
 static const sslExtensionBuilder clientHelloSendersSSL3[] = {
     { ssl_renegotiation_info_xtn, &ssl3_SendRenegotiationInfoXtn },
@@ -165,6 +166,7 @@ static const sslExtensionBuilder tls13_hrr_senders[] = {
     { ssl_tls13_key_share_xtn, &tls13_ServerSendHrrKeyShareXtn },
     { ssl_tls13_cookie_xtn, &tls13_ServerSendHrrCookieXtn },
     { ssl_tls13_supported_versions_xtn, &tls13_ServerSendSupportedVersionsXtn },
+    { ssl_tls13_encrypted_client_hello_xtn, &tls13_ServerSendHrrEchXtn },
     { 0, NULL }
 };
 
@@ -275,7 +277,7 @@ SSLExp_InstallExtensionHooks(PRFileDesc *fd, PRUint16 extension,
     return SECSuccess;
 }
 
-static sslCustomExtensionHooks *
+sslCustomExtensionHooks *
 ssl_FindCustomExtensionHooks(sslSocket *ss, PRUint16 extension)
 {
     PRCList *cursor;
@@ -322,6 +324,14 @@ ssl3_ExtensionAdvertised(const sslSocket *ss, PRUint16 ex_type)
                                   xtnData->numAdvertised, ex_type);
 }
 
+PRBool
+ssl3_ExtensionAdvertisedClientHelloInner(const sslSocket *ss, PRUint16 ex_type)
+{
+    const TLSExtensionData *xtnData = &ss->xtnData;
+    return arrayContainsExtension(xtnData->echAdvertised,
+                                  xtnData->echNumAdvertised, ex_type);
+}
+
 /* Go through hello extensions in |b| and deserialize
  * them into the list in |ss->ssl3.hs.remoteExtensions|.
  * The only checking we do in this point is for duplicates.
@@ -512,11 +522,21 @@ ssl3_HandleParsedExtensions(sslSocket *ss, SSLHandshakeType message)
          * do not have any response, so we rely on
          * ssl3_ExtensionAdvertised to return false on the server.  That
          * results in the server only rejecting any extension. */
-        if (!allowNotOffered && (extension->type != ssl_tls13_cookie_xtn) &&
-            !ssl3_ExtensionAdvertised(ss, extension->type)) {
-            (void)SSL3_SendAlert(ss, alert_fatal, unsupported_extension);
-            PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
-            return SECFailure;
+        if (!allowNotOffered && (extension->type != ssl_tls13_cookie_xtn)) {
+            if (!ssl3_ExtensionAdvertised(ss, extension->type)) {
+                SSL_TRC(10, ("Server sent xtn type=%d which is invalid for the CHO", extension->type));
+                (void)SSL3_SendAlert(ss, alert_fatal, unsupported_extension);
+                PORT_SetError(SSL_ERROR_RX_UNEXPECTED_EXTENSION);
+                return SECFailure;
+            }
+            /* If we offered ECH, we also check whether the extension is compatible with
+            * the Client Hello Inner. We don't yet know whether the server accepted ECH,
+            * so we only store this for now. If we later accept, we check this boolean
+            * and reject with an unsupported_extension alert if it is set. */
+            if (ss->ssl3.hs.echHpkeCtx && !ssl3_ExtensionAdvertisedClientHelloInner(ss, extension->type)) {
+                SSL_TRC(10, ("Server sent xtn type=%d which is invalid for the CHI", extension->type));
+                ss->ssl3.hs.echInvalidExtension = PR_TRUE;
+            }
         }
 
         /* Check that this is a legal extension in TLS 1.3 */
@@ -529,12 +549,23 @@ ssl3_HandleParsedExtensions(sslSocket *ss, SSLHandshakeType message)
                     if (allowNotOffered) {
                         continue; /* Skip over unknown extensions. */
                     }
-                /* Fall through. */
+                    /* RFC8446 Section 4.2 - Implementations MUST NOT send extension responses if
+                     * the remote endpoint did not send the corresponding extension request ...
+                     * Upon receiving such an extension, an endpoint MUST abort the handshake with
+                     * an "unsupported_extension" alert. */
+                    SSL_TRC(3, ("%d: TLS13: unknown extension %d in message %d",
+                                SSL_GETPID(), extension, message));
+                    tls13_FatalError(ss, SSL_ERROR_RX_UNEXPECTED_EXTENSION,
+                                     unsupported_extension);
+                    return SECFailure;
                 case tls13_extension_disallowed:
-                    SSL_TRC(3, ("%d: TLS13: unexpected extension %d in message %d",
+                    /* RFC8446 Section 4.2 - If an implementation receives an extension which it
+                     * recognizes and which is not specified for the message in which it appears,
+                     * it MUST abort the handshake with an "illegal_parameter" alert. */
+                    SSL_TRC(3, ("%d: TLS13: disallowed extension %d in message %d",
                                 SSL_GETPID(), extension, message));
                     tls13_FatalError(ss, SSL_ERROR_EXTENSION_DISALLOWED_FOR_VERSION,
-                                     unsupported_extension);
+                                     illegal_parameter);
                     return SECFailure;
             }
         }
@@ -629,7 +660,7 @@ ssl3_RegisterExtensionSender(const sslSocket *ss,
     return SECFailure;
 }
 
-static SECStatus
+SECStatus
 ssl_CallCustomExtensionSenders(sslSocket *ss, sslBuffer *buf,
                                SSLHandshakeType message)
 {
@@ -688,6 +719,7 @@ ssl_CallCustomExtensionSenders(sslSocket *ss, sslBuffer *buf,
         buf->len += len;
 
         if (message == ssl_hs_client_hello ||
+            message == ssl_hs_ech_outer_client_hello ||
             message == ssl_hs_certificate_request) {
             ss->xtnData.advertised[ss->xtnData.numAdvertised++] = hook->type;
         }
@@ -717,6 +749,7 @@ ssl_ConstructExtensions(sslSocket *ss, sslBuffer *buf, SSLHandshakeType message)
 
     /* Clear out any extensions previously advertised */
     ss->xtnData.numAdvertised = 0;
+    ss->xtnData.echNumAdvertised = 0;
 
     switch (message) {
         case ssl_hs_client_hello:
@@ -801,6 +834,9 @@ ssl_ConstructExtensions(sslSocket *ss, sslBuffer *buf, SSLHandshakeType message)
     }
 
     if (!PR_CLIST_IS_EMPTY(&ss->extensionHooks)) {
+        if (message == ssl_hs_client_hello && ss->opt.callExtensionWriterOnEchInner) {
+            message = ssl_hs_ech_outer_client_hello;
+        }
         rv = ssl_CallCustomExtensionSenders(ss, buf, message);
         if (rv != SECSuccess) {
             goto loser;
@@ -835,9 +871,6 @@ ssl_SendEmptyExtension(const sslSocket *ss, TLSExtensionData *xtnData,
 static unsigned int
 ssl_CalculatePaddingExtLen(const sslSocket *ss, unsigned int clientHelloLength)
 {
-    unsigned int recordLength = 1 /* handshake message type */ +
-                                3 /* handshake message length */ +
-                                clientHelloLength;
     unsigned int extensionLen;
 
     /* Don't pad for DTLS, for SSLv3, or for renegotiation. */
@@ -851,11 +884,11 @@ ssl_CalculatePaddingExtLen(const sslSocket *ss, unsigned int clientHelloLength)
      * the ClientHello doesn't have a length between 256 and 511 bytes
      * (inclusive). Initial ClientHello records with such lengths trigger bugs
      * in F5 devices. */
-    if (recordLength < 256 || recordLength >= 512) {
+    if (clientHelloLength < 256 || clientHelloLength >= 512) {
         return 0;
     }
 
-    extensionLen = 512 - recordLength;
+    extensionLen = 512 - clientHelloLength;
     /* Extensions take at least four bytes to encode. Always include at least
      * one byte of data if we are padding. Some servers will time out or
      * terminate the connection if the last ClientHello extension is empty. */
@@ -891,7 +924,9 @@ ssl3_EmplaceExtension(sslSocket *ss, sslBuffer *buf, PRUint16 exType,
     } else {
         tailLen = 0;
     }
-
+    if (exType == ssl_tls13_encrypted_client_hello_xtn) {
+        ss->xtnData.echXtnOffset = buf->len;
+    }
     rv = sslBuffer_AppendNumber(buf, exType, 2);
     if (rv != SECSuccess) {
         return SECFailure; /* Code already set. */
@@ -953,7 +988,7 @@ ssl3_MoveRemoteExtensions(PRCList *dst, PRCList *src)
     while (!PR_CLIST_IS_EMPTY(src)) {
         cur_p = PR_LIST_TAIL(src);
         PR_REMOVE_LINK(cur_p);
-        PR_APPEND_LINK(cur_p, dst);
+        PR_INSERT_LINK(cur_p, dst);
     }
 }
 
@@ -996,6 +1031,8 @@ ssl3_InitExtensionData(TLSExtensionData *xtnData, const sslSocket *ss)
         ++advertisedMax;
     }
     xtnData->advertised = PORT_ZNewArray(PRUint16, advertisedMax);
+    xtnData->echAdvertised = PORT_ZNewArray(PRUint16, advertisedMax);
+
     xtnData->peerDelegCred = NULL;
     xtnData->peerRequestedDelegCred = PR_FALSE;
     xtnData->sendingDelegCredToPeer = PR_FALSE;
@@ -1018,14 +1055,11 @@ ssl3_DestroyExtensionData(TLSExtensionData *xtnData)
         xtnData->certReqAuthorities.arena = NULL;
     }
     PORT_Free(xtnData->advertised);
+    PORT_Free(xtnData->echAdvertised);
     tls13_DestroyDelegatedCredential(xtnData->peerDelegCred);
 
-    /* ECH State */
-    SECITEM_FreeItem(&xtnData->innerCh, PR_FALSE);
-    SECITEM_FreeItem(&xtnData->echSenderPubKey, PR_FALSE);
-    SECITEM_FreeItem(&xtnData->echConfigId, PR_FALSE);
-    SECITEM_FreeItem(&xtnData->echRetryConfigs, PR_FALSE);
-    xtnData->echRetryConfigsValid = PR_FALSE;
+    tls13_DestroyEchXtnState(xtnData->ech);
+    xtnData->ech = NULL;
 }
 
 /* Free everything that has been allocated and then reset back to
