@@ -21,6 +21,8 @@
 #include "prenv.h"
 #include "prinit.h"
 
+#include <stdint.h>
+
 #ifdef DEBUG
 #define THREADMARK
 #endif /* DEBUG */
@@ -29,7 +31,7 @@
 #include "prthread.h"
 #endif /* THREADMARK */
 
-#if defined(XP_UNIX) || defined(XP_OS2) || defined(XP_BEOS)
+#if defined(XP_UNIX) || defined(XP_OS2)
 #include <stdlib.h>
 #else
 #include "wtypes.h"
@@ -119,6 +121,51 @@ PORT_ZAlloc(size_t bytes)
     return rv;
 }
 
+/* aligned_alloc is C11. This is an alternative to get aligned memory. */
+void *
+PORT_ZAllocAligned(size_t bytes, size_t alignment, void **mem)
+{
+    size_t x = alignment - 1;
+
+    /* This only works if alignment is a power of 2. */
+    if ((alignment == 0) || (alignment & (alignment - 1))) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+        return NULL;
+    }
+
+    if (!mem) {
+        return NULL;
+    }
+
+    /* Always allocate a non-zero amount of bytes */
+    *mem = PORT_ZAlloc((bytes ? bytes : 1) + x);
+    if (!*mem) {
+        PORT_SetError(SEC_ERROR_NO_MEMORY);
+        return NULL;
+    }
+
+    return (void *)(((uintptr_t)*mem + x) & ~(uintptr_t)x);
+}
+
+void *
+PORT_ZAllocAlignedOffset(size_t size, size_t alignment, size_t offset)
+{
+    PORT_Assert(offset < size);
+    if (offset > size) {
+        return NULL;
+    }
+
+    void *mem = NULL;
+    void *v = PORT_ZAllocAligned(size, alignment, &mem);
+    if (!v) {
+        return NULL;
+    }
+
+    PORT_Assert(mem);
+    *((void **)((uintptr_t)v + offset)) = mem;
+    return v;
+}
+
 void
 PORT_Free(void *ptr)
 {
@@ -152,9 +199,6 @@ PORT_Strdup(const char *str)
 void
 PORT_SetError(int value)
 {
-#ifdef DEBUG_jp96085
-    PORT_Assert(value != SEC_ERROR_REUSED_ISSUER_AND_SERIAL);
-#endif
     PR_SetError(value, 0);
     return;
 }
@@ -686,8 +730,6 @@ int
 NSS_PutEnv(const char *envVarName, const char *envValue)
 {
     SECStatus result = SECSuccess;
-    char *encoded;
-    int putEnvFailed;
 #ifdef _WIN32
     PRBool setOK;
 
@@ -696,22 +738,29 @@ NSS_PutEnv(const char *envVarName, const char *envValue)
         SET_ERROR_CODE
         return SECFailure;
     }
-#endif
-
-    encoded = (char *)PORT_ZAlloc(strlen(envVarName) + 2 + strlen(envValue));
+#elif defined(__GNUC__) && __GNUC__ >= 7
+    int setEnvFailed;
+    setEnvFailed = setenv(envVarName, envValue, 1);
+    if (setEnvFailed) {
+        SET_ERROR_CODE
+        return SECFailure;
+    }
+#else
+    char *encoded = (char *)PORT_ZAlloc(strlen(envVarName) + 2 + strlen(envValue));
     if (!encoded) {
         return SECFailure;
     }
     strcpy(encoded, envVarName);
     strcat(encoded, "=");
     strcat(encoded, envValue);
+    int putEnvFailed = putenv(encoded); /* adopt. */
 
-    putEnvFailed = putenv(encoded); /* adopt. */
     if (putEnvFailed) {
         SET_ERROR_CODE
         result = SECFailure;
         PORT_Free(encoded);
     }
+#endif
     return result;
 }
 
@@ -724,12 +773,30 @@ NSS_SecureMemcmp(const void *ia, const void *ib, size_t n)
 {
     const unsigned char *a = (const unsigned char *)ia;
     const unsigned char *b = (const unsigned char *)ib;
-    size_t i;
-    unsigned char r = 0;
+    int r = 0;
 
-    for (i = 0; i < n; ++i) {
-        r |= *a++ ^ *b++;
+    for (size_t i = 0; i < n; ++i) {
+        r |= a[i] ^ b[i];
     }
 
-    return r;
+    /* 0 <= r < 256, so -r has bit 8 set when r != 0 */
+    return 1 & (-r >> 8);
+}
+
+/*
+ * Perform a constant-time check if a memory region is all 0. The return value
+ * is 0 if the memory region is all zero.
+ */
+unsigned int
+NSS_SecureMemcmpZero(const void *mem, size_t n)
+{
+    const unsigned char *a = (const unsigned char *)mem;
+    int r = 0;
+
+    for (size_t i = 0; i < n; ++i) {
+        r |= a[i];
+    }
+
+    /* 0 <= r < 256, so -r has bit 8 set when r != 0 */
+    return 1 & (-r >> 8);
 }
